@@ -51,14 +51,15 @@ namespace arith {
     enum class hint_type {
         farkas_h,
         bound_h,
-        implied_eq_h    
+        implied_eq_h
     };
 
     struct arith_proof_hint : public euf::th_proof_hint {
-        hint_type                              m_ty;
-        unsigned m_lit_head, m_lit_tail, m_eq_head, m_eq_tail;
-        arith_proof_hint(hint_type t, unsigned lh, unsigned lt, unsigned eh, unsigned et):
-            m_ty(t), m_lit_head(lh), m_lit_tail(lt), m_eq_head(eh), m_eq_tail(et) {}
+        hint_type m_ty;
+        unsigned  m_num_le;
+        unsigned  m_lit_head, m_lit_tail, m_eq_head, m_eq_tail;
+        arith_proof_hint(hint_type t, unsigned num_le, unsigned lh, unsigned lt, unsigned eh, unsigned et):
+            m_ty(t), m_num_le(num_le), m_lit_head(lh), m_lit_tail(lt), m_eq_head(eh), m_eq_tail(et) {}
         expr* get_hint(euf::solver& s) const override;
     };
 
@@ -66,13 +67,14 @@ namespace arith {
         vector<std::pair<rational, literal>>   m_literals;
         svector<std::tuple<euf::enode*,euf::enode*,bool>> m_eqs;
         hint_type                              m_ty;
+        unsigned                               m_num_le = 0;
         unsigned                               m_lit_head = 0, m_lit_tail = 0, m_eq_head = 0, m_eq_tail = 0;
         void reset() { m_lit_head = m_lit_tail; m_eq_head = m_eq_tail; }
         void add(euf::enode* a, euf::enode* b, bool is_eq) {
-            if (m_eq_tail < m_eqs.size()) 
-                m_eqs[m_eq_tail] = std::tuple(a, b, is_eq); 
-            else 
-                m_eqs.push_back(std::tuple(a, b, is_eq)); 
+            if (m_eq_tail < m_eqs.size())
+                m_eqs[m_eq_tail] = { a, b, is_eq };
+            else
+                m_eqs.push_back({a, b, is_eq });
             m_eq_tail++; 
         }
     public:
@@ -82,6 +84,7 @@ namespace arith {
             m_ty = ty; 
             reset(); 
         }
+        void set_num_le(unsigned n) { m_num_le = n; }
         void add_eq(euf::enode* a, euf::enode* b) { add(a, b, true); }
         void add_diseq(euf::enode* a, euf::enode* b) { add(a, b, false); }
         void add_lit(rational const& coeff, literal lit) { 
@@ -94,7 +97,7 @@ namespace arith {
         std::pair<rational, literal> const& lit(unsigned i) const { return m_literals[i]; }
         std::tuple<enode*, enode*, bool> const& eq(unsigned i) const { return m_eqs[i]; }
         arith_proof_hint* mk(euf::solver& s) { 
-            return new (s.get_region()) arith_proof_hint(m_ty, m_lit_head, m_lit_tail, m_eq_head, m_eq_tail);
+            return new (s.get_region()) arith_proof_hint(m_ty, m_num_le, m_lit_head, m_lit_tail, m_eq_head, m_eq_tail);
         }
     };
 
@@ -105,11 +108,8 @@ namespace arith {
 
         struct scope {
             unsigned m_bounds_lim;
-            unsigned m_idiv_lim;
             unsigned m_asserted_qhead;
             unsigned m_asserted_lim;
-            unsigned m_underspecified_lim;
-            expr* m_not_handled;
         };
 
         class resource_limit : public lp::lp_resource_limit {
@@ -220,7 +220,7 @@ namespace arith {
         svector<std::pair<euf::th_eq, bool>>          m_delayed_eqs;
 
         literal_vector  m_asserted;
-        expr* m_not_handled{ nullptr };
+        expr* m_not_handled = nullptr;
         ptr_vector<app>        m_underspecified;
         ptr_vector<expr>       m_idiv_terms;
         vector<ptr_vector<api_bound> > m_use_list;        // bounds where variables are used.
@@ -325,6 +325,7 @@ namespace arith {
         void mk_bound_axiom(api_bound& b1, api_bound& b2);
         void mk_power0_axioms(app* t, app* n);
         void flush_bound_axioms();
+        void add_farkas_clause(sat::literal l1, sat::literal l2);
 
         // bounds
         struct compare_bounds {
@@ -475,8 +476,9 @@ namespace arith {
         arith_proof_hint_builder m_arith_hint;
 
         arith_proof_hint const* explain(hint_type ty, sat::literal lit = sat::null_literal);
-        arith_proof_hint const* explain_implied_eq(euf::enode* a, euf::enode* b);
-        void explain_assumptions();
+        arith_proof_hint const* explain_implied_eq(lp::explanation const& e, euf::enode* a, euf::enode* b);
+        arith_proof_hint const* explain_trichotomy(sat::literal le, sat::literal ge, sat::literal eq);
+        void explain_assumptions(lp::explanation const& e);
 
 
     public:
@@ -502,8 +504,8 @@ namespace arith {
         void finalize_model(model& mdl) override { DEBUG_CODE(dbg_finalize_model(mdl);); }
         void add_value(euf::enode* n, model& mdl, expr_ref_vector& values) override;
         bool add_dep(euf::enode* n, top_sort<euf::enode>& dep) override;
-        sat::literal internalize(expr* e, bool sign, bool root, bool learned) override;
-        void internalize(expr* e, bool redundant) override;
+        sat::literal internalize(expr* e, bool sign, bool root) override;
+        void internalize(expr* e) override;
         void eq_internalized(euf::enode* n) override;
         void apply_sort_cnstr(euf::enode* n, sort* s) override {}
         bool is_shared(theory_var v) const override;
