@@ -32,13 +32,10 @@ public:
     mpq(mpq &&) noexcept = default;
     mpq & operator=(mpq&&) = default;
     mpq & operator=(mpq const&) = delete;
-    void swap(mpq & other) { m_num.swap(other.m_num); m_den.swap(other.m_den); }
     mpz const & numerator() const { return m_num; }
     mpz const & denominator() const { return m_den; }
 };
 
-inline void swap(mpq & m1, mpq & m2) { m1.swap(m2); }
-
 template<bool SYNCH = true>
 class mpq_manager : public mpz_manager<SYNCH> {
     mpz m_tmp1;
